import './Shower.scss';
import Taro, { Component, Config } from '@tarojs/taro';
import { View, Image, Text } from '@tarojs/components';
import ShowerStart from '../../images/shower/ic_sel@2x.png';
import ShowerCantStart from '../../images/shower/ic_nor@2x.png';
import LineImg from '../../images/shower/img_nor@2x.png';
import ScanIcon from '../../images/shower/ic_saoma@2x.png';

import { shareHandle } from '@/common/shareMethod';
import {
  fetchShowerInfo,
  DeviceInfoResponse,
  startShowerEquipment,
  fetchUsingShowerInfo,
} from '@/api/shower';
import { connect } from '@tarojs/redux';
import { Customer } from '@/types/Customer/Customer';
import { ComponentClass } from 'react';
import { ResponseDataEntity } from '@/api';
import { connectSocket } from '@/api/socket';
import { SOCKET_URL } from '@/constants';

type PageStateProps = {
  userinfo: Customer;
};
type PageDispatchProps = {};

type IProps = PageStateProps & PageDispatchProps;

type PageState = {
  bluetoothState: boolean;
  code: string;
  isOnlyBluetooth: number | undefined;
  position: string;
  deviceId: string;
  task: Taro.SocketTask | null;
  timer: number | undefined;
};

interface Shower {
  props: IProps;
  state: PageState;
}

@connect(({ userinfo }) => ({
  userinfo,
}))
class Shower extends Component {
  config: Config = {
    navigationBarTitleText: '洗 浴',
  };

  constructor(props) {
    super(props);
    this.state = {
      bluetoothState: false,
      code: '',
      isOnlyBluetooth: undefined,
      position: '',
      deviceId: '',
      task: null,
      timer: undefined,
    };
  }

  onShareAppMessage = shareHandle;

  componentWillMount() {
    this.openBluetooth();
    this.connectDeviceSocket();
    this.checkUsingDevice();
  }

  componentWillUnmount() {
    this.closeBluetooth();
  }

  connectDeviceSocket() {
    connectSocket(SOCKET_URL).then(task => {
      this.setState({
        task: task,
      });
      task.onMessage(msg => {
        console.log('onMessage: ', msg);
        const data: string = msg.data;
        if (data.length > 100) {
          console.log('传递[]');
        } else if (data.length > 20) {
        } else {
        }
      });
    });
  }

  checkUsingDevice() {
    const { userinfo } = this.props;
    fetchUsingShowerInfo({
      customerId: userinfo.customerId,
    })
      .then(res => {
        const data: DeviceInfoResponse | null = res.data;
        if (data) {
          this.setState({
            code: data.code,
            isOnlyBluetooth: data.isOnlyBluetooth,
            position: data.position,
          });
        }
      })
      .catch(err => {
        console.error(err);
      });
  }

  openBluetooth() {
    Taro.openBluetoothAdapter()
      .then(res => {
        console.log(res);
        this.setState({
          bluetoothState: true,
        });
      })
      .catch(err => {
        console.error(err);
        Taro.onBluetoothAdapterStateChange(res => {
          console.log('adapterState changed, now is', res);
          if (res.available) {
            this.setState({
              bluetoothState: true,
            });
          }
        });
      });
  }

  checkBluetoothState() {
    const { bluetoothState } = this.state;
    return bluetoothState;
  }

  closeBluetooth() {
    Taro.closeBluetoothAdapter();
  }

  warnBluetoothOpen() {
    Taro.showModal({
      title: '警告',
      content: '为保障正常使用，请打开蓝牙功能',
    });
  }

  scanEquipment() {
    Taro.scanCode({
      onlyFromCamera: true,
      scanType: ['qrCode'],
    })
      .then(res => {
        console.log(res);
        const { result } = res;
        const { userinfo } = this.props;

        fetchShowerInfo({
          deviceCode: result,
          customerId: userinfo.customerId.toString(),
          campusId: userinfo.areaId,
        })
          .then((res: ResponseDataEntity<DeviceInfoResponse>) => {
            const data = res.data;
            this.setState({
              code: data.code,
              isOnlyBluetooth: data.isOnlyBluetooth,
              position: data.position,
            });
          })
          .catch(err => {
            console.error(err);
            Taro.showToast({
              title: err.msg || '请扫描正确的设备码',
              icon: 'none',
            });
          });
      })
      .catch(err => {
        console.error(err);
      });
  }

  startDevicesDiscovery() {
    if (this.checkBluetoothState()) {
      return Taro.startBluetoothDevicesDiscovery()
        .then(() => {
          const { code, isOnlyBluetooth } = this.state;
          return new Promise(resolve => {
            Taro.onBluetoothDeviceFound(res => {
              const devices = res.devices;
              if (devices[0].name === code) {
                let deviceId = devices[0].deviceId;
                Taro.stopBluetoothDevicesDiscovery();
                this.createConnection(deviceId);
                this.setState({
                  deviceId,
                });
                resolve(deviceId);
              }
            });
            if (isOnlyBluetooth) {
              setTimeout(() => {}, 10000);
            } else {
              setTimeout(() => {}, 5000);
            }
          });
        })
        .catch(err => {
          // Taro.stopBluetoothDevicesDiscovery();
          Taro.hideLoading();
          console.error('startBluetoothDevicesDiscovery: ', err);
          this.warnBluetoothOpen();
        });
    } else {
      this.warnBluetoothOpen();
      throw '未开启蓝牙';
    }
  }

  createConnection(deviceId: string) {
    console.log(deviceId);

    Taro.createBLEConnection({
      deviceId,
    });
    Taro.onBLEConnectionStateChange(res => {
      console.log(
        `device ${res.deviceId} state has changed, connected: ${res.connected}`,
      );
    });
  }

  startUseShower() {
    Taro.showLoading({
      title: '开启中',
      mask: true,
    });
    this.startDevicesDiscovery()
      .then(() => {
        const { code } = this.state;
        const { userinfo } = this.props;
        startShowerEquipment({
          deviceCode: code,
          customerId: userinfo.customerId.toString(),
          customerName: userinfo.customerName,
          customerPhone: userinfo.customerPhone,
        })
          .then(res => {
            console.log(res);
            Taro.showToast({
              title: '开启成功！',
            });
          })
          .catch(err => {
            console.error(err);
            Taro.showToast({
              title: err.msg || '开启失败！',
              icon: 'none',
            });
          });
      })
      .catch(err => {
        console.error(err);
        Taro.hideLoading();
      });
  }

  render() {
    const { code, position } = this.state;
    return (
      <View className='Shower'>
        <View className='equipment-info-box'>
          <View className='equipment-info'>
            {code ? (
              <View className='equipment-title matched'>已匹配设备</View>
            ) : (
              <View className='equipment-title'>未匹配设备</View>
            )}

            <Image className='equipment-line' src={LineImg} />
            {code ? (
              <View className='equipment-list'>
                <View className='equipment-item'>
                  <Text className='equipment-label'>设备编号：</Text>
                  <Text className='equipment-value'>{code}</Text>
                </View>
                <View className='equipment-item'>
                  <Text className='equipment-label'>设备位置：</Text>
                  <Text className='equipment-value'>{position}</Text>
                </View>
              </View>
            ) : (
              <View className='equipment-noCode'>点击左下角扫一扫完成匹配</View>
            )}
          </View>
        </View>
        <View className='btn-box'>
          <Image
            className='shower-scan-btn'
            src={ScanIcon}
            onClick={this.scanEquipment}
          />
          {code ? (
            <Image
              className='shower-start-btn'
              src={ShowerStart}
              onClick={this.startUseShower}
            />
          ) : (
            <Image className='shower-start-btn' src={ShowerCantStart} />
          )}
        </View>
      </View>
    );
  }
}

export default Shower as ComponentClass<PageStateProps, PageState>;
